#------------------------------------------------------------------------------
# Module      : D a t a . p m
#------------------------------------------------------------------------------
# Autor       : Christian Dhl
# Created     : 18. Januar 2005
# last change : 21. Januar 2005
# Task        : Data functions of GED
#------------------------------------------------------------------------------

package Graph::GED::Data;

use strict;
use warnings;

use Graph::GED::Core qw/$Animation $Graph %Konfig/;
use Graph::GED::Gui;
use Graph::GED::Logic;

#------------------------------------------------------------------------------
# Derive from Exporter:
#------------------------------------------------------------------------------
use base qw/Exporter/;
our @EXPORT_OK = qw//;


#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug = 2;


sub dump_data () {
    #--------------------------------------------------------------------------
    # sub          : d u m p _ d a t a
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Gibt die internen Daten auf dem Bildschirm aus.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 03.02.2003 - CD - In dump_data umbenannt.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    print "dump_data\n" if $Konfig{debug};

    print $Graph->n(), "\n";
    for my $i (0..$Graph->n-1) {
        print $Graph->ecke($i)->x(), ";",
              $Graph->ecke($i)->y(), "\n";
    }

    print $Graph->m(), "\n";
    for my $i (0..$Graph->m()-1) {
        print $Graph->kante($i)->a(), ";",
              $Graph->kante($i)->e(), ";",
              $Graph->kante($i)->form(), "\n";
    }
} # sub Graph::GED::Data::dump_data


sub save ($) {
    #--------------------------------------------------------------------------
    # sub          : s a v e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Interne Speicherroutine.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt.
    # 0.0.2 - 14.02.2003 - CD - Dateiname der History zugefgt.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.6 - 01.01.2005 - CD - Funktionalitt nach Graph2D::save_ged() aus-
    #                           gelagert.
    # 0.0.7 - 03.01.2005 - CD - nderungsmerker $Konfig{changed} wird zurck
    #                           gesetzt.
    # 0.0.8 - 16.01.2005 - CD - Graph2D::save() verwendet.
    #--------------------------------------------------------------------------

    my $dateiname = shift;

    print "save, Name = '$dateiname' <--------\n" if $Konfig{debug};

    if ($Graph->save($dateiname)) {
        $Konfig{statuszeile} = "Datei '$dateiname' gespeichert.";
        add($dateiname);
        $Konfig{changed} = ' ';
    }

} # sub Graph::GED::Data::save


sub load ($) {
    #--------------------------------------------------------------------------
    # sub          : l o a d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Interne Laderoutine.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 17.02.2003 - CD - Dateinamen in History gespeichert.
    # 0.0.3 - 18.03.2003 - CD - Neue Linientypenkrzel verwendet. Dafr werden
    #                           beim Laden die ausfhrlichen Formen in die
    #                           kurzen bersetzt, um alte Dateien richtig la-
    #                           den zu knnen.
    # 0.0.4 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.5 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.6 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    # 0.0.7 - 01.01.2005 - CD - Funktionalitt nach Graph2D::load_ged() aus-
    #                           gelagert.
    # 0.0.8 - 03.01.2005 - CD - nderungsmerker $Konfig{changed} wird zurck
    #                           gesetzt.
    # 0.0.9 - 16.01.2005 - CD - Graph2D::load() verwendet.
    #--------------------------------------------------------------------------

    my $dateiname = shift;

    print "load ($dateiname)\n" if $Konfig{debug};

    Graph::GED::Logic::aenderung();                  # Aktuellen Graphen in Historie sichern.

    if ($Graph->load($dateiname)) {
        $Konfig{statuszeile} = "Datei '$dateiname' geladen.";
        add($dateiname);              # neuen Dateiname zu Historie hinzufgen
        $Konfig{changed} = ' ';
    }

} # sub Graph::GED::Data::load


sub neue_datei () {
    #--------------------------------------------------------------------------
    # sub          : n e u e _ d a t e i
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht alles. Eventuell wird hier mal mehr gemacht, als nur
    #                clear aufzurufen, wenn man sich noch Dateinamen merkt o.a..
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf %Konfig.
    # 0.0.3 - 03.01.2005 - CD - nderungsmarker zurckgesetzt.
    # 0.0.4 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.5 - 18.01.2005 - CD - GUI-Funktionen nach Graph::GED::Gui verscho-
    #                           ben.
    #--------------------------------------------------------------------------

    print "neue_datei\n" if $Konfig{debug};
    Graph::GED::Gui::clear();
    $Graph->dateiname('');
    Graph::GED::Gui::set_title();
    $Konfig{changed} = ' ';
} # sub Graph::GED::Data::neue_datei


sub neue_animation () {
    #--------------------------------------------------------------------------
    # sub          : n e u e _ a n i m a t i o n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht alles.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 21.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "neue_animation\n" if $Konfig{debug};
    Graph::GED::Gui::clear();
    $Graph->dateiname('');
    $Animation->clear();
    $Animation->push_graph($Graph);
    $Konfig{animationskala}->configure(
        -to => $Animation->number_of_graphs()
    );    Graph::GED::Gui::set_title();
    $Konfig{changed} = ' ';
} # sub Graph::GED::Data::neue_animation


sub add ($) {
    #--------------------------------------------------------------------------
    # sub          : a d d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt den Dateinamen in die Liste ein und aktualisiert das
    #                Men.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 10.02.2004 - CD - Module in Namensraum Graph::GED:: verschoben.
    # 0.0.4 - 18.01.2005 - CD - Tk-Funktionen nach Graph::GED::Gui::Gui
    #                           verschoben.
    #--------------------------------------------------------------------------

    my $dateiname = shift;

    Graph::GED::FHist::add($dateiname);
    Graph::GED::Gui::add_file_menu();
} # sub Graph::GED::Data::add



1;


__END__

=head1 NAME

Graph::GED::Data.pm - a module with the data functions of GED

=head1 FUNCTIONS

=over

=item C<dump_data>

    dump_data

=item C<save>

    save

=item C<load>

    load

=item C<neue_datei>

    neue_datei

=item C<add>

    add

=back

=head1 COPYRIGHT

Copyright 2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
