#------------------------------------------------------------------------------
# Module      : L o g i c . p m
#------------------------------------------------------------------------------
# Autor       : Christian Dhl
# Created     : 18. Januar 2005
# last change : 19. Januar 2005
# Task        : Logic functions of GED
#------------------------------------------------------------------------------

package Graph::GED::Logic;

use strict;
use warnings;


use Graph::GED::Core qw/%Konfig $Graph @GitterX @GitterY $History/;
use Graph::GED::Data;
use Graph::GED::FHist;
use Graph::GED::Gui;
use Graph::GED::Historie;

#------------------------------------------------------------------------------
# Derive from Exporter:
#------------------------------------------------------------------------------
use base qw/Exporter/;
our @EXPORT_OK = qw//;


#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug = 0;


sub beenden () {
    #--------------------------------------------------------------------------
    # sub          : b e e n d e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Wird zum Ende des Programms aufgerufen.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 14.02.2003 - CD - History wird gespeichert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 06.02.2004 - CD - Statt exit wird tk_beenden aufgerufen.
    # 0.0.5 - 10.02.2004 - CD - Module in Namensraum Graph::GED:: verschoben.
    # 0.0.6 - 18.01.2005 - CD - GUI-Funktionen nach Graph::GED::Gui verscho-
    #                           ben.
    #                           Funktion nach Graph::GED::Logic verschoben.
    #--------------------------------------------------------------------------

    print "beenden\n" if $Konfig{debug};
    Graph::GED::Data::dump_data() if $Konfig{debug};

    Graph::GED::FHist::save();

    Graph::GED::Gui::tk_beenden();
} # sub Graph::GED::Logic::beenden


sub struktur_einfuegen ($) {
    #--------------------------------------------------------------------------
    # sub          : s t r u k t u r _ e i n f u e g e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt Kreise / Vollstndige Graphen ein
    # Parameter    : Strukturkennung, beginnt mt C ode K gefolgt von einer
    #                natrlichen Zahl > 2.
    #                Oder 'M' fr den Mittelpunkt.
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2004 - CD - Funktionalitt in die Klasse Graph2D ausge-
    #                           lagert.
    # 0.0.2 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    # 0.0.3 - 18.01.2005 - CD - GUI-Funktionen nach Graph::GED::Gui verscho-
    #                           ben.
    #                           Funktion nach Graph::GED::Logic verschoben.
    #--------------------------------------------------------------------------

    #----------------------------------------------------------------------
    # Aktuellen Graphen in Historie sichern:
    #----------------------------------------------------------------------
    aenderung();

    $Graph->struktur_einfuegen(@_);
    Graph::GED::Gui::refresh();
} # sub Graph::GED::Logic::struktur_einfuegen


sub raste_ein ($$) {
    #--------------------------------------------------------------------------
    # sub          : r a s t e _ e i n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Rastet ein Koordinatenpaar auf den nchsten Gitterlinien-
    #                kreuzungspunkt ein.
    # Parameter    : 1) X-Koordinate
    #                2) Y-Koordinate
    # Rckgabewert : (x-Koordinate, y-Koordinate)
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Funktion nach Graph::GED::Logic verschoben.
    #--------------------------------------------------------------------------

    my $x = shift;
    my $y = shift;

    print "x=$x, X:", @GitterX, "\n",
          "y=$y, Y:", @GitterY, "\n"
          if $Konfig{debug} > 1;

    # X-Koordinate bearbeiten:
    my $xindex = 0;
    my $xminab = abs($x-$GitterX[0]);
    for (1..$#GitterX) {
        if ($xminab > abs($x-$GitterX[$_])) {
            $xminab = abs($x-$GitterX[$_]);
            $xindex = $_;
        }
    }
    print "naechster Wert zu $x ist Index $xindex ($GitterX[$xindex])\n"
          if $Konfig{debug} > 1;

    # Y-Koordinate bearbeiten:
    my $yindex = 0;
    my $yminab = abs($y-$GitterY[0]);
    for (1..$#GitterY) {
        if ($yminab > abs($y-$GitterY[$_])) {
            $yminab = abs($y-$GitterY[$_]);
            $yindex = $_;
        }
    }
    print "naechster Wert zu $y ist Index $yindex ($GitterY[$yindex])\n"
          if $Konfig{debug} > 1;


    return ($GitterX[$xindex], $GitterY[$yindex]);
} # sub Graph::GED::Logic::raste_ein


sub erzeuge_liniengraphen () {
    #--------------------------------------------------------------------------
    # sub          : e r z e u g e _ l i n i e n g r a p h e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Erzeugt aus dem gegebenen Graphen seinen Liniengraphen.
    # Parameter    : keine
    # Rckgabewert : keiner (ein neuer Graph wird erzeugt)
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 07.02.2004 - CD - Waitcursor anzeigen whrend Berechnung und
    #                           refresh.
    # 0.0.3 - 08.02.2004 - CD - eigentliche Funktionalitt in Klasse Graph2D
    #                           ausgelagert.
    # 0.0.4 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    # 0.0.5 - 18.01.2005 - CD - GUI-Funktionen nach Graph::GED::Gui verscho-
    #                           ben.
    #                           Funktion nach Graph::GED::Logic verschoben.
    #--------------------------------------------------------------------------

    print "erzeuge_liniengraphen\n" if $Konfig{debug};


    $Konfig{mw}->Busy(-recurse => 1);

    #----------------------------------------------------------------------
    # Aktuellen Graphen in Historie sichern:
    #----------------------------------------------------------------------
    aenderung();

    #----------------------------------------------------------------------
    # Eigentlichen Liniengraphen erzeugen:
    #----------------------------------------------------------------------
    $Graph->erzeuge_liniengraphen();

    #----------------------------------------------------------------------
    # Weitere Eigenschaften zurcksetzen:
    #----------------------------------------------------------------------
    $Graph->kantenform('k_ins');
    $Konfig{startecke}  = -1;
    $Konfig{zx}         = 0;
    $Konfig{zy}         = 0;

    Graph::GED::Gui::refresh();

    $Konfig{mw}->Unbusy();
} # sub Graph::GED::Logic::erzeuge_liniengraphen


sub rueckgaengig () {
    #--------------------------------------------------------------------------
    # sub          : r u e c k g a e n g i g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : nderungen rckgngig machen.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - GUI-Funktionen nach Graph::GED::Gui verscho-
    #                           ben.
    #                           Funktion nach Graph::GED::Logic verschoben.
    # 0.0.3 - 19.01.2005 - CD - Historie jetzt in $Graph::GED::Core::History.
    #--------------------------------------------------------------------------

    print "rueckgaengig\n" if $Konfig{debug};

    my ($g, $alt, $neu) = $History->rueckgaengig($Graph);
    $Graph->clear();
    $Graph = undef;
    $Graph = $g;
    Graph::GED::Gui::historie_schalter_aktualisieren($alt, $neu);
    Graph::GED::Gui::refresh();

} # sub Graph::GED::Logic::rueckgaengig


sub wiederholung () {
    #--------------------------------------------------------------------------
    # sub          : w i e d e r h o l u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Rckgngig wieder rckgngig machen...
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - GUI-Funktionen nach Graph::GED::Gui verscho-
    #                           ben.
    #                           Funktion nach Graph::GED::Logic verschoben.
    # 0.0.3 - 19.01.2005 - CD - Historie jetzt in $Graph::GED::Core::History.
    #--------------------------------------------------------------------------

    print "wiederholung\n" if $Konfig{debug};

    $Graph->clear();
    $Graph = undef;
    my ($g, $alt, $neu) = $History->wiederholung();
    $Graph = $g;
    Graph::GED::Gui::historie_schalter_aktualisieren($alt, $neu);
    Graph::GED::Gui::refresh();

} # sub Graph::GED::Logic::rueckgaengig


sub aenderung () {
    #--------------------------------------------------------------------------
    # sub          : a e n d e r u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Daten werden gleich gendert, aktuelle Version in
    #                Historie sichern.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    # 0.0.2 - 03.01.2005 - CD - nderungsmerker $Konfig{changed} wird gesetzt.
    # 0.0.3 - 18.01.2005 - CD - GUI-Funktionen nach Graph::GED::Gui verscho-
    #                           ben.
    #                           Funktion nach Graph::GED::Logic verschoben.
    # 0.0.3 - 19.01.2005 - CD - Historie jetzt in $Graph::GED::Core::History.
    #--------------------------------------------------------------------------

    print "aenderung\n" if $Konfig{debug};

    my ($alt, $neu) = $History->objekt_eintragen($Graph);
    Graph::GED::Gui::historie_schalter_aktualisieren($alt, $neu);
    $Konfig{changed} = '*';

} # sub Graph::GED::Logic::aenderung


1;


__END__

=head1 NAME

Graph::GED::Logic.pm - a module with the logical functions of GED

=head1 FUNCTIONS

=over

=item C<beenden>

    Terminates the programm.

=item C<struktur_einfuegen>

    Insertes a structure (circle or complete graph or point in the middle.

=item C<raste_ein>

    Snap into the grid.

=item C<erzeuge_liniengraphen>

    Create the line graph.
    
=item C<rueckgaengig>

    Take back.

=item C<wiederholung>

    Redo.

=item C<aenderung>

    Graph will change after this call.

=back

=head1 COPYRIGHT

Copyright 2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
